/* QMRFAttachment.java
 * Author: Nina Jeliazkova
 * Date: Feb 24, 2007 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package ambit.data.qmrf;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ambit.io.XMLException;

public class QMRFAttachment  implements InterfaceQMRF {
    protected static final String attribute_url = "url";
    protected static final String attribute_filetype = "filetype";
    protected static final String attribute_description = "description";
    protected String name = "document";
    protected String url;
    protected String filetype;
    protected String description;
    
    public QMRFAttachment(String elementID,String url, String filetype, String description) {
        this(elementID);

        setUrl(url);
        setFiletype(filetype);
        setDescription(description);
    }    
    public QMRFAttachment(String elementID) {
        super();
        setName(elementID);
        setUrl("");
        setFiletype("");
        setDescription("");
    }
    
    public synchronized String getFiletype() {
        return filetype;
    }
    public synchronized void setFiletype(String filetype) {
        if (filetype == null) this.filetype = "text";
        else this.filetype = filetype;
    }
    public synchronized String getUrl() {
        return url;
    }
    public synchronized void setUrl(String url) {
        if (url == null) this.url = "";
        else this.url = url;
    }

    public void fromXML(Element xml) throws XMLException {
        setFiletype(xml.getAttribute(attribute_filetype));
        setUrl(xml.getAttribute(attribute_url));
        setDescription(xml.getAttribute(attribute_description));
    }

    public Element toXML(Document document) throws XMLException {
        Element element = document.createElement(getName());
        element.setAttribute(attribute_url,url);
        element.setAttribute(attribute_filetype,filetype);
        element.setAttribute(attribute_description,description);        
        return element;
    }

    @Override
    public String toString() {
        return url + " (" + filetype + ")";
    }
    @Override
    public boolean equals(Object obj) {
        QMRFAttachment a = (QMRFAttachment) obj;
        return name.equals(a.getName()) && url.equals(a.getUrl()) && filetype.equals(a.getFiletype());
    }
    public void setName(String name) {
        this.name = name;
        
    }
    public String getName() {
        return name;
    }
    public synchronized String getDescription() {
        return description;
    }
    public synchronized void setDescription(String description) {
        this.description = description;
    }
}
