/*
Copyright Ideaconsult (C) 2005-2007 
Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

package chemCast.application;

import java.awt.Container;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.util.ArrayList;

import javax.swing.ActionMap;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import ambit.data.descriptors.DescriptorFactory;
import ambit.data.descriptors.DescriptorsHashtable;
import ambit.io.batch.DefaultBatchStatistics;
import ambit.io.batch.IBatchStatistics;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.Endpoint;
import chemCast.similarity.data.Endpoints;
import chemCast.similarity.data.SimilarityModel;
import chemCast.similarity.data.SimilaritySubsets;
import chemCast.ui.actions.ConfiguredTrainingSetOpenAction;
import chemCast.ui.actions.CreateSubsetAction;
import chemCast.ui.actions.DescriptorsAction;
import chemCast.ui.actions.FileNewAction;
import chemCast.ui.actions.FileNewTrainingAction;
import chemCast.ui.actions.FileOpenAction;
import chemCast.ui.actions.FileSaveAction;
import chemCast.ui.actions.ImportDescriptorsAction;
import chemCast.ui.actions.SelectFieldsAction;
import chemCast.ui.actions.SimilarityCalculationAction;
import chemCast.ui.actions.SubsetsAction;
import chemCast.ui.actions.TrainingSetOpenAction;
import chemCast.ui.actions.ViewModeAction;
import chemCast.ui.actions.ViewSimilarityModelsAction;
import chemCast.ui.wizard.SimilarityWizard;



public class ChemCASTModule  {
    public static final String[] datasetName = {"Training set","Test set"};
    public static final int DATA_TRAINING = 0;
    public static final int DATA_TEST = 1;
    public static final int DATA_COUNT = 2;
    protected String defaultDir = "";
    protected SimilaritySubsets[] datasets;
    
    /*
    protected Hashtable<SimilarityDef,ISimilarityProcessor>[] processors;
    
    protected ISimilarityProcessor[] selectedProcessor;
    */
    protected Endpoint endpoint = null;
    //protected SimilarityDef similarity = null;
    protected DescriptorsHashtable descriptors;
    
    
    protected PropertyChangeSupport propertyChangeSupport;
    protected IBatchStatistics batchStatistics;
    
    
    public ChemCASTModule() {
        super();
        propertyChangeSupport = new PropertyChangeSupport(this);
        datasets = new SimilaritySubsets[DATA_COUNT];
        
        for (int i=0; i < DATA_COUNT;i++) 
            datasets[i] = new SimilaritySubsets(datasetName[i],datasetName[i]);
        
        descriptors =  DescriptorFactory.createDescriptorsList();
        //similarity = new SimilarityDef();
        //subsets = new Subsets();
        batchStatistics = new DefaultBatchStatistics();
        defaultDir = System.getProperty("user.dir");
    }
    public void close() throws Exception {
        for (int i=0; i < DATA_COUNT;i++) datasets[i].clear();
        
    }


    public synchronized SimilaritySubsets getDataset(int index) {
        return datasets[index];
    }
    public static Endpoints readEndpoints() {
        File file = new File("ChemCAST_default.xml");
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(false);
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(file);
            NodeList nodes = doc.getElementsByTagName(Endpoints.p_endpoints);
            return new Endpoints((Element)nodes.item(0));
        } catch (Exception x) {
            x.printStackTrace();
            return null;
        }
    }
    
    public synchronized Endpoint getEndpoint() {
        return endpoint;
    }
    
    public static ChemCastDescriptorsList readDescriptors() {
        File file = new File("ChemCAST_default.xml");
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(false);
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(file);
            NodeList nodes = doc.getElementsByTagName(ChemCastDescriptorsList.p_descriptors);
            return new ChemCastDescriptorsList((Element)nodes.item(0));
        } catch (Exception x) {
            x.printStackTrace();
            return null;
        }
    }    
    public ArrayList<String> getProperties() {
        ArrayList<String> list = new ArrayList<String>();
        if (endpoint != null) {
            list.add(endpoint.getTrainingData().getDatasetResultField().toString());
            if (endpoint.getSimilarityMethods() != null)
                for (int i=0; i < endpoint.getSimilarityMethods().getList().size();i++)
                    list.add(endpoint.getSimilarityMethods().get(i).toString());

        }
        /*
        if (getSubsets() != null)
        	for (int i=0; i <  getSubsets().size();i++)
        		list.add(getSubsets().get(i).getTag());
        */
        return list;
    }
    public synchronized void setEndpoint(Endpoint endpoint) {
        for (int i=0; i < DATA_COUNT;i++) datasets[i].clear();
        propertyChangeSupport.firePropertyChange(Endpoint.p_endpoint, this.endpoint,endpoint);
        this.endpoint = endpoint;
    }

    /*
    public SimilarityDef getSimilarity() {
        return similarity;
    }
    public void setSimilarity(SimilarityModel similarity) throws Exception {
        for (int i=0; i < DATA_COUNT;i++)
            datasets[i].setSimilarity(similarity);
    }
    */
    public synchronized PropertyChangeSupport getPropertyChangeSupport() {
        return propertyChangeSupport;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }
     
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(propertyName,listener);
    }
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }
    
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(propertyName,listener);
    }

    protected void createTrainingDatasetActions(ActionMap actions, JFrame mainFrame) {
        actions.put("New",new FileNewTrainingAction(this,mainFrame,"New",DATA_TRAINING));
    	actions.put("Open", new TrainingSetOpenAction(this,mainFrame,"Open",DATA_TRAINING));    	
        actions.put("Predefined", new ConfiguredTrainingSetOpenAction(this,mainFrame,"Predefined training sets",DATA_TRAINING));
        actions.put("Save", new FileSaveAction(this,mainFrame,"Save",DATA_TRAINING));
    }
    protected void createViewTrainingDatasetActions(ActionMap actions, JFrame mainFrame) {
        actions.put("View train",new ViewModeAction(this,mainFrame,"View",DATA_TRAINING));
        actions.put("Fields train",new SelectFieldsAction(this,mainFrame,"Fields",DATA_TRAINING));
        
    }
    protected void createViewTestDatasetActions(ActionMap actions, JFrame mainFrame) {
        actions.put("View test",new ViewModeAction(this,mainFrame,"View",DATA_TEST));
        actions.put("Fields test",new SelectFieldsAction(this,mainFrame,"Fields",DATA_TEST));
    }        
    public void createTestDatasetActions(ActionMap actions,  JFrame mainFrame) {
        actions.put("Test New",new FileNewAction(this,mainFrame,"New",DATA_TEST));
        actions.put("Test Open",new FileOpenAction(this,mainFrame,"Open",DATA_TEST));
        actions.put("Test Save",new FileSaveAction(this,mainFrame,"Save",DATA_TEST));
    } 
    protected void createTrainingSimilarityActions(ActionMap actions,  JFrame mainFrame) {
        /*
        actions.put("select Similarity within the training set",
                new SelectSimilarityAction(this,mainFrame,"Select similarity",DATA_TRAINING));
                */
        actions.put("Similarity within the training set",
                new SimilarityCalculationAction(this,mainFrame,"Similarity to the training set",DATA_TRAINING,DATA_TRAINING));
        actions.put("Similarity within the test set",
                new SimilarityCalculationAction(this,mainFrame,"Similarity to the test set",DATA_TRAINING,DATA_TEST));
        actions.put("ViewTraining",new ViewSimilarityModelsAction(this,mainFrame,"Explore similarities",DATA_TRAINING));        
    }
    protected void createTrainingDescriptorsActions(ActionMap actions,  JFrame mainFrame) {
        
        actions.put("Descriptors import training",new ImportDescriptorsAction(this,mainFrame,"Import descriptors",DATA_TRAINING));
        actions.put("Descriptors training",new DescriptorsAction(this,mainFrame,"Calculate descriptors",DATA_TRAINING));
    }
    protected void createTestDescriptorsActions(ActionMap actions,  JFrame mainFrame) {
        actions.put("Descriptors test",new DescriptorsAction(this,mainFrame,"Calculate descriptors",DATA_TEST));
        actions.put("Descriptors import test",new ImportDescriptorsAction(this,mainFrame,"Import descriptors",DATA_TEST));
    }        
    protected void createTestSimilarityActions(ActionMap actions,  JFrame mainFrame) {
        actions.put("Similarity to the training set",
                new SimilarityCalculationAction(this,mainFrame,"Similarity to the training set",DATA_TEST,DATA_TRAINING));        
        actions.put("Similarity within the test set",
                new SimilarityCalculationAction(this,mainFrame,"Similarity to the test set",DATA_TEST,DATA_TEST));
        actions.put("ViewTest",new ViewSimilarityModelsAction(this,mainFrame,"Explore similarities",DATA_TEST));        
        
    }
    protected void createTrainingSubsetActions(ActionMap actions,  JFrame mainFrame) {
        actions.put("Subset training",new SubsetsAction(this,mainFrame,"Select groups",DATA_TRAINING));
        actions.put("New Subset training",new CreateSubsetAction(this,mainFrame,"New group",DATA_TRAINING));
        
        
    }
    protected void createTestSubsetActions(ActionMap actions,  JFrame mainFrame) {
        actions.put("Subset test",new SubsetsAction(this,mainFrame,"Select groups",DATA_TEST));
        actions.put("New Subset test",new CreateSubsetAction(this,mainFrame,"New group",DATA_TEST));        
    }        
    
    /*
    public static void createTrainingDescriptorsActions(ActionMap actions, Object userData, JFrame mainFrame) {
        actions.put("Import descriptors",createTODOAction("Import descriptors",mainFrame));
        actions.put("Calculate descriptors",new ContainerDescriptorCalculationAction(userData,mainFrame,false)) ;
        actions.put("Select best descriptors",createTODOAction("Select best descriptors",mainFrame));
    }

    public static void createTestDescriptorsActions(ActionMap actions, Object userData, JFrame mainFrame) {
        actions.put("Test Import descriptors",createTODOAction("Import descriptors",mainFrame));
        actions.put("Test Calculate descriptors",new ContainerDescriptorCalculationAction(userData,mainFrame,true));
        actions.put("Test Select best descriptors",createTODOAction("Select best descriptors",mainFrame));
    }
    public static void createTrainingSubsetActions(ActionMap actions, Object userData, JFrame mainFrame) {
        actions.put("Predefined groups",createTODOAction("Predefined groups",mainFrame));
        actions.put("Manual selection",createTODOAction("Manual selection",mainFrame));
        actions.put("Clustering",createTODOAction("Clustering",mainFrame));
    }
    public static void createTestSubsetActions(ActionMap actions, Object userData, JFrame mainFrame) {
        actions.put("Predefined groups",createTODOAction("Predefined groups",mainFrame));
        actions.put("Manual selection",createTODOAction("Manual selection",mainFrame));
        actions.put("Clustering",createTODOAction("Clustering",mainFrame));

    }
    public static void createVisualActions(ActionMap actions, Object userData, JFrame mainFrame) {
        actions.put("Scatter plot",createTODOAction("Scatter plot",mainFrame));
        actions.put("Histogram",createTODOAction("Histogram",mainFrame));
        actions.put("Similariy matrix",createTODOAction("Similariy matrix",mainFrame));
    } 
    */   
    public synchronized SimilarityModel getSelectedProcessor(int dataset) {
        return datasets[dataset].getSelectedProcessor();
    }
    public synchronized String getDefaultDir() {
        return defaultDir;
    }
    public synchronized void setDefaultDir(String defaultDir) {
        this.defaultDir = defaultDir;
    }    
    
    public boolean selectSimilarity(Container parent, SimilaritySubsets subsets){
        try {
            if (subsets.size()==0) return false;
            if (getEndpoint()==null) return false;
	        SimilarityWizard wizard = new SimilarityWizard(parent,
	                getEndpoint().getSimilarityMethods(),
                    //subsets.getSimilarity()
	                null,subsets);
	        wizard.getDialog().setTitle("Select similarity method");
	        
	        if (wizard.showModalDialog() == 0) { 
                subsets.setSimilarity(wizard.getSimilarityMethod());
                return true;
            }    
            else return false;
        } catch (Exception x) {
            x.printStackTrace();
                JOptionPane.showMessageDialog(parent,x.getMessage());
                return false;
        } 
    }
    public synchronized IBatchStatistics getBatchStatistics() {
        return batchStatistics;
    }
    public synchronized void setBatchStatistics(IBatchStatistics batchStatistics) {
        this.batchStatistics = batchStatistics;
    }
    public synchronized SimilaritySubsets[] getDatasets() {
        return datasets;
    }

}
