/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.data;

import java.util.Iterator;
import java.util.Map;

import org.openscience.cdk.interfaces.IChemObject;

import ambit.log.AmbitLogger;
import ambit.processors.IAmbitProcessor;

public class ProcessorFilter<T extends IAmbitProcessor> extends AbstractFilter implements IObjectFilter {
	protected static AmbitLogger logger = new AmbitLogger(ProcessorFilter.class);
	protected T processor;
	public ProcessorFilter() {
		this(null);
	}	
	public ProcessorFilter(T processor) {
		super();
		setProcessor(processor);
	}

	public boolean accept(IChemObject object) {
		try {
			return processor.process(object) != null;
		} catch (Exception x) {
			x.printStackTrace();
			return false;
		}
	}

	public T getProcessor() {
		return processor;
	}

	public void setProcessor(T processor) {
		this.processor = processor;
	}
	@Override
	public String toString() {
		return processor.toString();
	}
	@Override
	public void setProperty(String key, Object value) {
		super.setProperty(key, value);
		if ("processor".equals(key)) {
			try {
				Class c = Class.forName(value.toString());
				T o = (T)c.newInstance();
				setProcessor(o);
			} catch (Exception x) {
				logger.error(x);
			}		
		}
		
	}
	public void setProperties(Map properties) {
		
		Iterator i = properties.keySet().iterator();
		while (i.hasNext()) {
			Object key = i.next();
			setProperty(key.toString(), properties.get(key));
		}
	}	
	protected boolean isProcessorValid(Object o) {
		return o instanceof IAmbitProcessor;
	}
}


