/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.similarity.data;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.qsar.DescriptorValue;
import org.openscience.cdk.qsar.result.IDescriptorResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import ambit.data.IAmbitEditor;
import ambit.data.literature.ReferenceFactory;
import ambit.exceptions.AmbitException;
import ambit.processors.IAmbitProcessor;
import ambit.processors.IAmbitResult;

public class ChemCastDescriptorsList extends ChemCastObject implements
		IChemCastList, IAmbitProcessor {
	/**
	 * 
	 */
	private static final long serialVersionUID = -6490394909575643112L;
	protected ArrayList<ChemCastDescriptor> list;
    public static String p_descriptors="descriptors";

	public ChemCastDescriptorsList(Element element) throws Exception {
        this();
        setXMLElement(element);
        Collections.sort(list);
	}

	public ChemCastDescriptorsList() {
		super();
		list = new ArrayList<ChemCastDescriptor>();
	}

	@Override
    public Element getXMLElement(Document document) {
        return ChemCastObjectFactory.getXMLElement(list, document, p_descriptors);
    }

	@Override
	public void setXMLElement(Element element) throws Exception {
        if (element.getNodeName().equals(p_descriptors)) {
            NodeList nodes = element.getElementsByTagName(ChemCastDescriptor.p_descriptor);
            for (int i=0; i < nodes.getLength(); i++) {
                ChemCastDescriptor s = new ChemCastDescriptor("default",ReferenceFactory.createEmptyReference());
                s.setXMLElement((Element)nodes.item(i));
                add(s);
            }
        }
	}
    public void add(ChemCastDescriptor descriptor) {
        firePropertyChange("add", null, descriptor);
        list.add(descriptor);
    }
	public List<? extends IChemCastObject> getList() {
		return list;
	}

	public int compareTo(Object o) {
		return 0;
	}
    public void clear() {
        list.clear();
        super.clear();
        firePropertyChange("clear", null, this);
    }
    public String toString() {
        return "Descriptors"; 
    }
    public ChemCastDescriptor get(int index) {
    	return list.get(index);
    }
    public Object process(Object object) throws AmbitException {
    	if (object instanceof IAtomContainer) {
	    	for (int i=0; i < list.size();i++)
	    		if (get(i).isEnabled())
	    			try {
	    				DescriptorValue value = get(i).calculate((IAtomContainer)object);
	    				IDescriptorResult result = value.getValue();
	    				((IAtomContainer)object).setProperty(get(i).getTag(), result);
	    			} catch (CDKException x) {
	    				x.printStackTrace();
	    			}
	    	return object;
    	}	
	    else throw new AmbitException("Not an IAtomContainer! ");			
    }
    
    public void setResult(IAmbitResult result) {
    	
    }
    public IAmbitResult getResult() {

    	return null;
    }
    public void close() {
    	
    }
    public IAmbitResult createResult() {
    	return null;
    }
    public IAmbitEditor getEditor() {
    	return null;
    }
    public int indexOf(ChemCastDescriptor d) {
        return list.indexOf(d);
    }
    public void selectAll(boolean selected) {
    	for (ChemCastDescriptor d : list)
    		d.setEnabled(selected);
    	firePropertyChange("clear", null, this);
    	
    }
}


