/* NNFingerprintSimilaritySubsets.java
 * Author: Nina Jeliazkova
 * Date: Feb 4, 2007 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.util.BitSet;

import org.openscience.cdk.interfaces.IAtomContainer;

import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.structure.FingerprintGenerator;
import ambit.similarity.FingerprintDistance;
import ambit.similarity.IDistanceFunction;
import ambit.similarity.RRelieFFAttributeSelection;

public class NNFingerprintSimilaritySubsets extends
        NearestNeighborsSimilaritySubsets<BitSet> {
    protected FingerprintGenerator generator;
    /**
     * 
     */
    private static final long serialVersionUID = 5920387758012582805L;
    public NNFingerprintSimilaritySubsets() {
        this(10);
    }    
    public NNFingerprintSimilaritySubsets(int knn) {
        this(new FingerprintDistance(),knn);
    	//this(new BinaryKernelDistance(),knn);
    }
    public NNFingerprintSimilaritySubsets(IDistanceFunction<BitSet> distanceFunction, int knn) {
        super(distanceFunction, knn);
        generator = new FingerprintGenerator();
        getNearestNeighborsSearch().setLowestDistanceIsClosest(false);
        /*
        setAttributeSelection(new RRelieFFAttributeSelection<BitSet>() {

			@Override
			protected Object getAttribute(BitSet attributes, int attribute_index) {
				return readAttribute(attributes, attribute_index);
			}

			@Override
			protected double getMaxAttribute(int attribute_index) {
				return readMaxAttribute(attribute_index);
			}
			@Override
			protected double getMinAttribute(int attribute_index) {
				return readMinAttribute(attribute_index);
			}
			public BitSet transformToSelected(BitSet attributes) {
				if (isEvaluated() && (weights!= null)) {
					int j =0;
					BitSet newAttributes = new BitSet();
					for (int i=0; i < weights.length;i++)
						if (weights[i]>getThreshold()) {
							j++;
							newAttributes.set(j,attributes.get(i));
						}	
					System.out.println(j + " " + newAttributes.size());
					return newAttributes;
				} else return attributes;			
			}

        });             
        */
    }
    @Override
    protected BitSet getSelectedAttributes(BitSet attributes) {
    	return attributeSelection.transformToSelected(attributes);
    }
    @Override
    protected BitSet calculateComparableProperty(Object object)
            throws AmbitException {
        IAtomContainer ac = ((IAtomContainer)object);
        Object ae = ac.getProperty(AmbitCONSTANTS.Fingerprint);
        if ((ae != null) && (!"".equals(ae)) && (ae instanceof BitSet)) 
            return (BitSet) ae;
        generator.process(ac);
        ae = ac.getProperty(AmbitCONSTANTS.Fingerprint);
        if (ae instanceof BitSet) return (BitSet) ae; else return null;
    }
    @Override
    protected BitSet readComparableProperty(IAtomContainersList list, int index) throws AmbitException {
        Object ae = list.getProperty(index, AmbitCONSTANTS.Fingerprint);
        if ((ae != null) && (!"".equals(ae)) && (ae instanceof BitSet)) 
            return (BitSet) ae;        
        else return null;
    }
    @Override
    protected void writeComparableProperty(IAtomContainersList list, int index, BitSet property) throws AmbitException {
        list.setProperty(index, AmbitCONSTANTS.Fingerprint, property);
        
    }
    
    @Override
    protected int getNumberOfAttributes() {
    	return generator.getFPLength();
    }
	@Override
	protected Object readAttribute(BitSet attributes, int attribute_index) {
    	return new Boolean(attributes.get(attribute_index));
	}
	@Override
	protected double readMaxAttribute(int attribute_index) {
		return 1;
	}
	@Override
	protected double readMinAttribute(int attribute_index) {
		return 0;
	}

}
