/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.similarity.data;

import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.interfaces.IChemObject;
import org.openscience.cdk.qsar.model.QSARModelException;

import toxTree.core.IDecisionMethod;
import toxTree.core.Introspection;
import verhaar.VerhaarScheme;
import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.processors.toxtree.ToxTreeProcessor;
import ambit.similarity.DefaultSimilarityProcessor;
import ambit.similarity.IClassificationModel;
import ambit.similarity.StringValuesHistogram;
import chemCast.application.StatusBar;
import chemCast.weka.ISubsetsModel;

public class ToxTreeSimilarity extends DefaultSimilarityProcessor<String> implements
		ISubsetsModel, IClassificationModel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3715072057986566061L;
	public static String p_decisionMethod = "decisionMethod";
	protected  Subsets datasets;
	protected ToxTreeProcessor processor;
	

	public ToxTreeSimilarity() {
		super();
		try {
			processor = new ToxTreeProcessor(new VerhaarScheme());
			setSimilarityProperty(processor.getMethod().toString());
			setPairwiseSimilarityProperty(processor.getMethod().toString()+".pairwise");
		} catch (Exception x) {
			processor = null;
		}
		setPredicting(false);
	}

	public Subsets getDatasets() {
		return datasets;
	}

	public void setDatasets(Subsets datasets) {
		this.datasets = datasets;

	}

	public float predict(IAtomContainersList object1, int index1,
			IAtomContainersList object2, int index2) throws AmbitException {
		String sp = getSimilarityProperty();
		Object result1 = object1.getProperty(index1, sp);
		if ((result1 == null) || "".equals(result1)) {
			Object o1 = processor.process(object1.getAtomContainer(index1));
			result1 = ((IChemObject) o1).getProperty(sp);
			object1.setProperty(index1, sp, result1);
		}
		Object result2 = object2.getProperty(index2, sp);
		if ((result2 == null) || "".equals(result2)) {
			Object o2 = processor.process(object2.getAtomContainer(index2));
			result2 = ((IChemObject) o2).getProperty(sp);
			object2.setProperty(index2, sp, result2);
		}
		if (result1.equals(result2)) return 1; else return 0;
		
	}

	public void build() throws QSARModelException {
		// no need to build
		//histogram.clear();
        setTrainingPrefix(datasets.toString());
        setPredicting(true);
        setDatasets(null);
	}

    public void predict() throws QSARModelException {
        //histogram.clear();
        if (modelStatistics != null) {
        	modelStatistics.clear(datasets.toString());
        	modelStatistics.setTitle(getSimilarityProperty());
        }
        datasets.firePropertyChange(StatusBar.Message_Status,null,"Prediction phase: " + toString() + " " + datasets.toString());
        String sp = getSimilarityProperty();
        String spe = getSimilarityProperty() + "#explanation";
        for (int s=0; s < datasets.size();s++)
            if (datasets.get(s).isEnabled()) 
            try {
                IAtomContainersList list = datasets.get(s).getData();
                if (list == null) continue;
                for (int i=0; i < list.getAtomContainerCount();i++) 
                    try {
                        datasets.firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/list.getAtomContainerCount()));
                        IAtomContainer a = list.getAtomContainer(i);
                        processor.process(a);
                        
                        Object result = a.getProperty(processor.getMethod().toString());
                        list.setProperty(i, sp, result);
                        
                        if (modelStatistics == null) {
                        	modelStatistics = new StringValuesHistogram(getSimilarityProperty());
                        }
                        modelStatistics.addPoint(datasets,datasets.get(s).toString(), result);                        
                        modelStatistics.addValue(datasets,datasets.get(s).toString(), result.toString());

                    } catch (Exception x) {
                        datasets.firePropertyChange(StatusBar.Message_Error,null,x);
                        list.setProperty(i, sp, x.toString());
                    }
                    
            } catch (Exception x) {
                x.printStackTrace();
            }
            setDatasets(null);
    }

	@Override
	public String toString() {
		if (processor!=null)
			return processor.toString();
		else return "";
	}

	public String getClassificationProperty() {
		return getSimilarityProperty();
	}
	public void setClassificationProperty(String classificationProperty) {
		//setSimilarityProperty(classificationProperty);
	}
	public void setParameter(Object parameter, Object value) {
		if (parameter.equals(p_classification_property)) 
			setClassificationProperty(value.toString());
		else if (parameter.equals(p_decisionMethod)) {
			Object method = Introspection.loadCreateObject(value.toString());
			if ((method != null) && (method instanceof IDecisionMethod))
				processor.setMethod((IDecisionMethod)method);
		}
	}
	public Object getParameter(Object parameter) {
		if (parameter.equals(p_classification_property)) 
			return getClassificationProperty();
		else if (parameter.equals(p_decisionMethod)) 
			return processor.getMethod().getClass().getName();		
		else return null;
	}
    public String[] getProperties() {
	    	return new String[]{ 
	    			getSimilarityProperty()
	    			};
    }
    /*
    @Override
    public IAmbitEditor getEditor() {
        return processor.getEditor();
    }
    */
}


