/* BigFileTest.java
 * Author: Nina Jeliazkova
 * Date: Sep 28, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test;

import java.awt.Dimension;
import java.io.File;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JOptionPane;

import junit.framework.TestCase;

import org.openscience.cdk.DefaultChemObjectBuilder;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.interfaces.IChemObject;
import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;
import org.openscience.cdk.nonotify.NoNotificationChemObjectBuilder;
import org.openscience.cdk.templates.MoleculeFactory;

import ambit.io.RandomAccessReader;
import ambit.misc.AmbitCONSTANTS;
import ambit.similarity.FingerprintSimilarity;
import chemCast.data.FilteredMoleculesFile;
import chemCast.data.MoleculesFile;
import chemCast.data.RandomAccessSubsetTableModel;
import chemCast.data.RangePropertyFilter;
import chemCast.similarity.data.DatasetDef;
import chemCast.ui.MoleculesTable;

public class BigFileTest extends TestCase {
	//int n = 5000000;
	int n = 500;
    public BigFileTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }
    /**
     * This tests how many bitsets can fit in memory.
     * So far (1GB RAM)
     * <pre>
     * 1494660 empty bitsets
     * 660356 half empty  bs.set(0,511);
     * 401123 full  bs.set(0,1023);
     * 
     * </pre>
     */
    public void testBitsets() {
        ArrayList<BitSet> list = new ArrayList<BitSet>();
        try {
           
            for (int i=0; i < n; i++) {
                BitSet bs = new BitSet();
                //bs.set(0,511);
                bs.set(0,1023);
                list.add(bs);
                System.out.println((i+1));
                //System.out.print(' ');
                //System.out.println(bs.toString());
            }
            assertEquals(n,list.size());
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    }
    /**
     * This tests how many double[] arrays can fit in memory.
     * So far (1GB RAM)
     * <pre>
     *  80626 double[200]
     *  32631 double[100]
     *  156775 double[50]
     *  363825 double[20]
     *  660788 double[10]
     * 
     * </pre>
     */
    public void testArrays1() {
        ArrayList<double[]> list = new ArrayList<double[]>();
        try {
           
            int d = 100;
            for (int i=0; i < n; i++) {
                
                list.add(new double[d]);
                System.out.println((i+1));
            }
            assertEquals(n,list.size());
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    } 
    /**
     * <pre>
     * new double[1000000][100]  Out of memory
     * new double[1000000][100]  Out of memory
     * 
     * new double[500000][10] OK
     * new double[500000][20] Out of memory
     * new double[500000][50] Out of memory
     * new double[500000][100] Out of memory
     * 
     * new double[250000][10] OK
     * new double[250000][20] OK
     * new double[250000][50] Out of memory
     * new double[250000][100] Out of memory
     * </pre>
     */
    public void testMatrix() {
        double[][] data = new double[n][50];
    }
    /**
     * This tests how many Double[] arrays can fit in memory.
     * So far (1GB RAM)
     * <pre>
     *  16434 Double[200]
     *  32631 Double[100]
     *  64638 Double[50]
     *  156910 Double[20]
     *  297868 Double[10]
     * 
     * </pre>
     * Conclusion: 3 to 7 times worse than double[] (as expected ...)
     */
    public void testArrays() {
        ArrayList<Double[]> list = new ArrayList<Double[]>();
        try {
           
            int d = 10;
            for (int i=0; i < n; i++) {
                Double[] data = new Double[d];
                for (int k=0; k< d; k++)
                    data[k] = new Double(k);
                
                list.add(data);
                System.out.println((i+1));
            }
            assertEquals(n,list.size());
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    }
    /**
     * This tests how many hashtables can fit in memory.
     * So far (1GB RAM)
     * <pre>
     * 658723 empty hashtables
     * 69524 hashtables containing 10 pairs <String,Double> each; 
     * 93077 hashtables containing 10 pairs <Integer,Double> each; 
     * 
     * 10258 hashtables containing 100 pairs <Integer,Double> each; 
     * 7476 hashtables containing 100 pairs <String,Double> each; 
     * 
     * 3594 hashtables containing 200 pairs <String,Double> each;
     * 5159 hashtables containing 200 pairs <Integer,Double> each;
     * 
     * </pre>
     */
    public void testHashtable() {
        ArrayList<Hashtable> list = new ArrayList<Hashtable>();
        try {
           
            for (int i=0; i < n; i++) {
                Hashtable t = new Hashtable();
                for (int k=0; k < 200; k++)
                    t.put(new Integer(k),new Double(k));
                    //t.put(Integer.toString(k),new Double(k));
                list.add(t);
                System.out.println((i+1));
                //System.out.print(' ');
                //System.out.println(bs.toString());
            }
            assertEquals(n,list.size());
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    }
    /**
     * This tests how many {@link IAtomContainer}s can fit in memory.
     * So far (1GB RAM)
     * <pre>
     *
     * Molecules without properties
     * 1485  molecules created by MoleculeFactory.makeAlkane(100)
     * 2969  molecules created by MoleculeFactory.makeAlkane(50)
     * 4973  small molecules created by MoleculeFactory.makeAlkane(30)
     * 7433 small molecules created by MoleculeFactory.makeAlkane(20)
     * 14885 small molecules created by MoleculeFactory.makeAlkane(10) 
     * 
     * Molecules with properties - 10 pairs <String,Double> each;
     * 1454   molecules created by MoleculeFactory.makeAlkane(100)
     * 2848    molecules created by MoleculeFactory.makeAlkane(50)
     * 4643     molecules created by MoleculeFactory.makeAlkane(30)
     * 6718     molecules created by MoleculeFactory.makeAlkane(20)
     * 12266     molecules created by MoleculeFactory.makeAlkane(10)
     *
     * Molecules with properties - 50 pairs <String,Double> each;
     * 1350   molecules created by MoleculeFactory.makeAlkane(100)
     * 2481    molecules created by MoleculeFactory.makeAlkane(50)
     * 3734     molecules created by MoleculeFactory.makeAlkane(30)
     * 4966     molecules created by MoleculeFactory.makeAlkane(20)
     * 7443     molecules created by MoleculeFactory.makeAlkane(10)
     * 
     * Molecules with properties - 200 pairs <Integer,Double> each;
     * 1154   molecules created by MoleculeFactory.makeAlkane(100)
     * 1885    molecules created by MoleculeFactory.makeAlkane(50)
     * 2527     molecules created by MoleculeFactory.makeAlkane(30)
     * 3050     molecules created by MoleculeFactory.makeAlkane(20)
     * 3831     molecules created by MoleculeFactory.makeAlkane(10)
     * 
     * Molecules with properties - 200 pairs <String,Double> each;
     * 1052   molecules created by MoleculeFactory.makeAlkane(100)
     * 1629    molecules created by MoleculeFactory.makeAlkane(50)
     * 2086     molecules created by MoleculeFactory.makeAlkane(30)
     * 2421     molecules created by MoleculeFactory.makeAlkane(20)
     * 2887     molecules created by MoleculeFactory.makeAlkane(10)
     * </pre>
     * Conclusion: storing a set of about 1000 (average) molecules in memory is feasible, don't rely on more.
     * Ideally, don't store more than few hundred molecules at once ... an average application has a lot more things to store beside molecules...
     */
    public void testMolecules() {
        ArrayList<IAtomContainer> list = new ArrayList<IAtomContainer>();
        try {
           
            for (int i=0; i < n; i++) {
                IAtomContainer a = MoleculeFactory.makeAlkane(100);
                
                for (int k=0; k < 50; k++)
                    //a.setProperty(new Integer(k),new Double(k));
                    a.setProperty(Integer.toString(k),new Double(k));
                     
                list.add(a);
                System.out.println((i+1));
                //System.out.print(' ');
                //System.out.println(bs.toString());
            }
            assertEquals(n,list.size());
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    }
    /**
     * One more test how many {@link IAtomContainer}s can fit in memory.
     * So far (1GB RAM)
     * <pre>
     *
     * Molecules with a single property <String,double[10]>
     * 1478  molecules created by MoleculeFactory.makeAlkane(100)
     * 2941  molecules created by MoleculeFactory.makeAlkane(50)
     * 4894  small molecules created by MoleculeFactory.makeAlkane(30)
     * 7257 small molecules created by MoleculeFactory.makeAlkane(20)
     * 14201 small molecules created by MoleculeFactory.makeAlkane(10) 
     * 
     * Molecules with a single property <String,Double[10]>
     * 1474  molecules created by MoleculeFactory.makeAlkane(100)
     * 2925  molecules created by MoleculeFactory.makeAlkane(50)
     * 4851  small molecules created by MoleculeFactory.makeAlkane(30)
     * 7179 small molecules created by MoleculeFactory.makeAlkane(20)
     * 13840 small molecules created by MoleculeFactory.makeAlkane(10)
     * 
     * </pre>
     *  
     * <pre>
     * Molecules with a single property <String,double[200]>
     * 1432  molecules created by MoleculeFactory.makeAlkane(100)
     * 2755  molecules created by MoleculeFactory.makeAlkane(50)
     * 4404  small molecules created by MoleculeFactory.makeAlkane(30)
     * 6220 small molecules created by MoleculeFactory.makeAlkane(20)
     * 10711 small molecules created by MoleculeFactory.makeAlkane(10) 
     * 
     * Molecules with a single property <String,Double[200]>
     * 1361  molecules created by MoleculeFactory.makeAlkane(100)
     * 2505  molecules created by MoleculeFactory.makeAlkane(50)
     * 3799  small molecules created by MoleculeFactory.makeAlkane(30)
     * 5084 small molecules created by MoleculeFactory.makeAlkane(20)
     * 7715 small molecules created by MoleculeFactory.makeAlkane(10)
     * 
     *  Conclusion: Not much difference between double[200] and Double[200], especially for bigger molecules.
     *  Definitely better than hashtable of 200 pairs <String,Double> for small molecules, comparable for big molecules.
     * </pre>
     */
    public void testMoleculesArrayProperty() {
        ArrayList<IAtomContainer> list = new ArrayList<IAtomContainer>();
        try {
           
            int d = 10;
            for (int i=0; i < n; i++) {
                IAtomContainer a = MoleculeFactory.makeAlkane(100);
                /*
                Double[] data = new Double[d];
                for (int k=0; k< d; k++)
                    data[k] = new Double(k);
                a.setProperty("Array",data);    
                  */
                a.setProperty("Array",new double[d]);
                list.add(a);
                System.out.println((i+1));
            }
            assertEquals(n,list.size());
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    }
    public void testIterator() {
    	try {
    		File file = new File("data/LLNA.sdf");

    		MoleculesFile m = new MoleculesFile(file, DefaultChemObjectBuilder.getInstance());
			BitSet bs = new BitSet();
			bs.set(1,1023);
    		m.setProperty(1,AmbitCONSTANTS.Fingerprint,bs);
    		assertEquals(bs,m.getProperty(1,AmbitCONSTANTS.Fingerprint));
    		
    		int record = 0;
    		while (m.hasNext()) {
	    		Hashtable props = m.getProperties(record);
	    		Enumeration e = props.keys();
	    		while (e.hasMoreElements()) {
	    			Object key = e.nextElement();
	    			//Object value = props.get(key);
	    			m.setProperty(record,key,"test");
	    			
	    		}
	    		record++;
    		}
    		assertEquals("test",m.getProperty(1,AmbitCONSTANTS.Fingerprint));
    		m.close();
    	} catch (Exception x) {
    		x.printStackTrace();
    		fail();
    	}
    }
    public void testProcessing() {
    	try {
    		File file = new File("data/LLNA.sdf");

    		MoleculesFile m = new MoleculesFile(file, DefaultChemObjectBuilder.getInstance());
    		FingerprintSimilarity fp = new FingerprintSimilarity();
            System.out.println(m.getSelectedIndex());
            //TODO skips the first record ... fix it
    		while (m.hasNext()) {
    			Object o = fp.process(m.next());
                int record = m.getSelectedIndex();
                System.out.println(record);
                m.setProperty(record, AmbitCONSTANTS.Fingerprint, 
                        ((IChemObject)o).getProperty(AmbitCONSTANTS.Fingerprint));
                assertNotNull(m.getProperty(record,AmbitCONSTANTS.Fingerprint));
                

    			
    		}
            assertNotNull(m.getProperty(0,AmbitCONSTANTS.Fingerprint));
    		assertNotNull(m.getProperty(1,AmbitCONSTANTS.Fingerprint));

            fp.close();
            //prediction phase
            m.first();
            int count =0;
            double testRangeMin = 0.1;
            double testRangeMax = 0.2;
    		while (m.hasNext()) {
    			Object o = fp.process(m.next());
                int record = m.getSelectedIndex();
                System.out.println(record);
                
                Object d = ((IChemObject)o).getProperty(fp.getSimilarityProperty());
                m.setProperty(record, fp.getSimilarityProperty(),d); 
                        
                assertNotNull(m.getProperty(record,fp.getSimilarityProperty()));
                
                if (d instanceof Number) {
                	if ((((Number) d).doubleValue() >= testRangeMin) && (((Number) d).doubleValue() <= testRangeMax)) count++;   
                }
    			
    		}
            fp.close();
            
            FilteredMoleculesFile mf = new FilteredMoleculesFile(m, new RangePropertyFilter(fp.getSimilarityProperty(),testRangeMin,testRangeMax));
            assertTrue(mf.getAtomContainerCount() > 0);
            assertEquals(count,mf.getAtomContainerCount());
    		m.close();
    	} catch (Exception x) {
    		x.printStackTrace();
    		fail();
    	}
    }
    public void test() {
        try {
            File file = new File("data/LLNA_3D.sdf");
            //File file = new File("data/200406EINECS.sdf");
            //File file = new File("data/test.einecs.sdf");
            //File file = new File("E:/nina/LRI_Ambit/Chemical Databases/NCI/nciopen_3D_fixed.sdf");
            
            IReaderListener listener = new IReaderListener() {
              public void processIOSettingQuestion(IOSetting setting) {
                    // TODO Auto-generated method stub
                    
                }
              public void frameRead(ReaderEvent event) {
                  int records = ((RandomAccessReader) event.getSource()).size();
                  int record = ((RandomAccessReader) event.getSource()).getCurrentRecord();
                  if  (record ==0)
                      if ((records % 1000) == 0) {
                          System.out.println(records);
                      } else {}
                  else {
                      System.out.print(record);
                      System.out.print('/');
                      System.out.println(records);                      
                  }
                  event =null;
                
            }
            };

            MoleculesFile mf  =new MoleculesFile(file, NoNotificationChemObjectBuilder.getInstance()); 
            mf.getPropertyTranslator().put("CasRN","CAS #");
            mf.getPropertyTranslator().put("Names","CHEMICAL NAME");
            
            assertNotNull(mf.getProperty(0,"CasRN"));
            assertNotNull(mf.getProperty(0,"Names"));
            assertNotNull(mf.getProperty(0,"ambit.data.descriptors.MaximumDiameterDescriptor"));
            mf.getPropertyTranslator().put("maxD","ambit.data.descriptors.MaximumDiameterDescriptor");
            assertNotNull(mf.getProperty(0,"maxD"));
            DatasetDef subset = new DatasetDef();
            subset.setData(mf);
            
            
            MoleculesTable pane = new  MoleculesTable(
                    new RandomAccessSubsetTableModel(subset),
                    new Dimension(150,150));
            pane.setSpreadsheet(false);
            JOptionPane.showMessageDialog(null,pane);
            mf.close();
        } catch (Exception x) {
        	x.printStackTrace();
            fail();
        }
    }
    public void testLookup() {
        try {
            File file = new File("data/LLNA.sdf");

            MoleculesFile m = new MoleculesFile(file, DefaultChemObjectBuilder.getInstance());
            int index = m.indexOf("CAS #", "3326-32-7");
            assertTrue(index>0);
            int index1 = m.indexOf("LLNA EC3 %", new Double("0.140000"));
            assertTrue(index1>0);
            assertEquals(index,index1);
            m.close();
        } catch (Exception x) {
            
        }
    }    

}
