/* NearestNeighborsSimilaritySubsetsTest.java
 * Author: Nina Jeliazkova
 * Date: Feb 4, 2007 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test;

import junit.framework.TestCase;

import org.openscience.cdk.SetOfAtomContainers;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.templates.MoleculeFactory;

import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.MoleculesIterator;
import chemCast.similarity.data.NNAtomEnvironmentSimilaritySubsets;
import chemCast.similarity.data.NNFingerprintSimilaritySubsets;
import chemCast.similarity.data.NearestNeighborsSimilaritySubsets;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;

public class NearestNeighborsSimilaritySubsetsTest extends TestCase {

    public NearestNeighborsSimilaritySubsetsTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }
    public void testAtomEnvironments() {
        NNAtomEnvironmentSimilaritySubsets nns = new NNAtomEnvironmentSimilaritySubsets();
        IAtomContainersList a = new MoleculesIterator();
        for (int i=6; i < 12; i++) {
            IAtomContainer a1 = MoleculeFactory.makeAlkane(i);
            a.add(a1);
        }
        try {
            Subset d = new Subset();
            d.setData(a);
            Subsets s = new Subsets();
            s.add(d);
            runSubsetTest(nns,s);
        } catch (Exception x) {
            fail(x.getMessage());
        }

    }    
    public void testFingerprints() {
        NNFingerprintSimilaritySubsets nns = new NNFingerprintSimilaritySubsets(5) {
        	@Override
        	public String getResultField() {
        		return "Result";
        	}
        }
        ;
        IAtomContainersList a = new MoleculesIterator();
        for (int i=6; i < 18; i++) {
        	IAtomContainer a1;
            if (i>10) {
            	a1 = MoleculeFactory.makeAlkane(i);
            	a1.setProperty("Result", "Big");
            } else { 
            	a1 = MoleculeFactory.makeCyclohexene();
            	a1.setProperty("Result", "Small");
            }	
            a.add(a1);
        }
        try {
            Subset d = new Subset();
            d.setData(a);
            Subsets s = new Subsets();
            s.add(d);
            runSubsetTest(nns,s);
        } catch (Exception x) {
            fail(x.getMessage());
        }

    }
    public void runTest(NearestNeighborsSimilaritySubsets nns, SetOfAtomContainers a) {

        try {
            nns.setPredicting(false);
            nns.buildInitialize();
            for (int i=0; i < a.getAtomContainerCount(); i++) 
                try {
                    nns.process(a.getAtomContainer(i));
                } catch (Exception x) {
                    fail(x.getMessage());
                }
            nns.buildCompleted();
            
            assertEquals(a.getAtomContainerCount(),nns.getNearestNeighborsSearch().size());
            
            nns.setPredicting(true);
            System.out.println(nns);
            for (int i=0; i < a.getAtomContainerCount(); i++) 
                try {
                    nns.process(a.getAtomContainer(i));
                    System.out.println(a.getAtomContainer(i).getProperty(nns.getProperties()[0]));

                } catch (Exception x) {
                    x.printStackTrace();
                    fail(x.getMessage());
                }
            
        } catch (Exception x) {
            
        }
    }
    public void runSubsetTest(NearestNeighborsSimilaritySubsets nns, Subsets datasets) {

        try {
            nns.setDatasets(datasets);
            nns.build();
            //assertEquals(a.getAtomContainerCount(),nns.getNearestNeighborsSearch().size());
            nns.setDatasets(datasets);
            nns.predict();
            
        } catch (Exception x) {
            x.printStackTrace();
            fail(x.getMessage());
        }
    }

}
