/* TestChart.java
 * Author: Nina Jeliazkova
 * Date: Sep 29, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test.ui;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JOptionPane;
import javax.swing.JSplitPane;

import junit.framework.TestCase;

import org.openscience.cdk.DefaultChemObjectBuilder;
import org.openscience.cdk.aromaticity.HueckelAromaticityDetector;
import org.openscience.cdk.interfaces.IMolecule;
import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;
import org.openscience.cdk.templates.MoleculeFactory;

import ambit.data.molecule.IAtomContainersList;
import ambit.io.RandomAccessReader;
import ambit.ui.data.RandomAccessFileTableModel;
import chemCast.data.FilteredMoleculesFile;
import chemCast.data.IObjectFilter;
import chemCast.data.MoleculesFile;
import chemCast.data.ObjectPropertyFilter;
import chemCast.data.RandomAccessSubsetTableModel;
import chemCast.data.RangePropertyFilter;
import chemCast.similarity.data.IChemCastObject;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;
import chemCast.ui.MoleculesTable;
import chemCast.ui.MyChart;

/**
 * TODO add description
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Sep 29, 2006
 */
public class TestChart extends TestCase {

    /**
     * @param arg0
     */
    public TestChart(String arg0) {
        super(arg0);
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }
    public void testProcessing() {
        try {
            File file = new File("data/LLNA.sdf");
            IReaderListener listener = new IReaderListener() {
            	public void processIOSettingQuestion(IOSetting arg0) {
            		// TODO Auto-generated method stub
            		
            	}
                public void frameRead(ReaderEvent event) {
                    int records = ((RandomAccessReader) event.getSource()).size();
                    int record = ((RandomAccessReader) event.getSource()).getCurrentRecord();
                    if  (record ==0)
                        if ((records % 1000) == 0) {
                            System.out.println(records);
                        } else {}
                    else {
                        System.out.print(record);
                        System.out.print('/');
                        System.out.println(records);                      
                    }
                    event =null;
                  
              }
            };
            
            final MoleculesFile mf = new MoleculesFile(file,DefaultChemObjectBuilder.getInstance());
            mf.getReader().addChemObjectIOListener(listener);
            for (int i=0; i < mf.getAtomContainerCount();i++)
            	mf.setProperty(i, "similarity", Math.random());
            
            mf.getProperty(0, "Vehicle1");
            assertTrue(mf.isBuffered(0, "Vehicle1"));
            
            mf.getProperty(0, "BLABLA");
            assertTrue(mf.isBuffered(0, "BLABLA"));
            
            
            ArrayList props = new ArrayList();
            props.add("Vehicle1");
            props.add("Potency_numeric");
            props.add("LLNA EC3 %");
            props.add("CAS #");
            props.add("CHEMICAL NAME");
            
            
            mf.bufferedProperties(props); 
            
            Iterator e = props.iterator();
            while (e.hasNext()) {
                Object key = e.next();
                for (int i=0; i < mf.getAtomContainerCount();i++) 
                    if(!mf.isBuffered(i,key)) 
                        System.out.println(i + " " + key);    
                    
                
            }
            
            
            FilteredMoleculesFile m1 = new FilteredMoleculesFile(mf,new ObjectPropertyFilter("Vehicle1","AOO")); 
            
            
            IMolecule mol = MoleculeFactory.makeAlkane(5);
            HueckelAromaticityDetector.detectAromaticity(mol);
            /*
            FilteredMoleculesFile m4 = new FilteredMoleculesFile(mf,new ProcessorFilter(
            		new SubstructureSearchProcessor(mol)
            		));
            assertTrue(m4.getAtomContainerCount()>0);
            */
            final Subsets list = new Subsets();
            list.setType("training");
            
            Subset s2 = new Subset();
            
            s2.setFilterClass("chemCast.data.PropertyFilter");
            s2.setProperty(ObjectPropertyFilter.p_tag,"Vehicle1");
            s2.setProperty(ObjectPropertyFilter.p_value, "Acetone");
            s2.setParentDataset(mf,null);

            IObjectFilter pf1 = s2.createFilter();
            ObjectPropertyFilter pf2 = new ObjectPropertyFilter("Vehicle1","Acetone");
            assertEquals(pf1,pf2);
            FilteredMoleculesFile m2 = new FilteredMoleculesFile(mf,pf2);
            assertEquals(s2.getData().getAtomContainerCount(),m2.getAtomContainerCount());
            assertTrue(s2.getData().getAtomContainerCount()>0);
            for (int i=0; i < m2.getAtomContainerCount();i++) {
            	assertEquals("Acetone",m2.getProperty(i,"Vehicle1"));
            }            
            
            m2.close();
            m2 = null;
            list.add(s2);
            
            Subset s3 = new Subset();
            
            s3.setFilterClass("chemCast.data.RangePropertyFilter");
            s3.setProperty(RangePropertyFilter.p_minValue, "2");
            s3.setProperty(RangePropertyFilter.p_maxValue, "3");
            s3.setProperty(RangePropertyFilter.p_tag,"Potency_numeric");
            s3.setParentDataset(mf,null);
            
            RangePropertyFilter rp = new RangePropertyFilter("Potency_numeric",2,3);
            IObjectFilter newFilter = s3.createFilter();
            assertEquals(rp,newFilter);
            FilteredMoleculesFile m3 = new FilteredMoleculesFile(mf,rp);
            assertTrue(m3.getAtomContainerCount()>0);
            
            for (int i=0; i < m3.getAtomContainerCount();i++) {
            	Object value = m3.getProperty(i,"Potency_numeric");
            	double d;
            	try {
	            	if (value instanceof Number) d = ((Number) value).doubleValue();
	            	else d = Double.parseDouble(value.toString());
            		assertTrue((d>=2) && (d<=3));	
            	} catch (Exception x) {
            		x.printStackTrace();
            	}
            }	
            IAtomContainersList c3 = s3.getData();
            for (int i=0; i < c3.getAtomContainerCount();i++) {
            	Object value = c3.getProperty(i,"Potency_numeric");
            	double d;
            	try {
	            	if (value instanceof Number) d = ((Number) value).doubleValue();
	            	else d = Double.parseDouble(value.toString());
	            	System.out.println(i);
            		assertTrue((d>=2) && (d<=3));	
            	} catch (Exception x) {
            		x.printStackTrace();
            	}
            }	            
            assertEquals(c3.getAtomContainerCount(),m3.getAtomContainerCount());
            list.add(s3);
            Subset s1 = new Subset();
            s1.setData(m1);
            list.add(s1);
            /*
            Subset s4 = new Subset();
            s4.setData(m4);
            list.add(s4);
            */
            
            MyChart panel = new MyChart(new Subsets[] {list,list},
            		"LLNA EC3 %",
            	    "Potency_numeric",
            		"similarity","CAS #",
            		"CHEMICAL NAME",null);
            panel.addPropertyChangeListener(list.toString(),
            		new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    
                        System.out.println("was " +evt.getOldValue());
                        System.out.println("new " + evt.getNewValue());
                    
                }
            });
            Subset smf = new Subset();
            smf.setParentDataset(mf,null);
            smf.setData(mf);
            //assertTrue(m3.getAtomContainerCount()>0);
            RandomAccessSubsetTableModel model = new RandomAccessSubsetTableModel(s3);
            
            MoleculesTable grid = new  MoleculesTable(
                    model,
                    new Dimension(150,150));
            grid.setSpreadsheet(false);

            
            PropertyChangeSupport ps = new PropertyChangeSupport(panel);
            //ps.addPropertyChangeListener(panel);
            ps.addPropertyChangeListener(grid);
            ps.firePropertyChange(IChemCastObject.p_properties, null, "\"LLNA  Sis\"_2");
            ps.firePropertyChange(IChemCastObject.p_properties, "LLNA EC3 %", null);
            
            panel.addPropertyChangeListener(grid);
            JSplitPane p = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,grid,panel);
            
            JOptionPane.showMessageDialog(null,p,"",JOptionPane.PLAIN_MESSAGE);
            mf.close();
            list.clear();
            
            /*
            
            file = new File("data/EPAFHM_v2a_617_1Mar05.sdf");
            reader = new RandomAccessSDFReader(file, DefaultChemObjectBuilder.getInstance());
            reader.addChemObjectIOListener(listener);
            
            final MoleculesFile mf1 = new MoleculesFile(reader);
            m1 = new FilteredMoleculesFile(mf1,new PropertyFilter("MOA","REACTIVE")); 
            m2 = new FilteredMoleculesFile(mf1,new PropertyFilter("MOA","NARCOSIS I"));
            m3 = new FilteredMoleculesFile(mf1,new PropertyFilter("MOA","ACHE"));
            
            final List<IAtomContainersList> list1 = new ArrayList<IAtomContainersList>();
            list1.add(m1);
            list1.add(m2);
            list1.add(m3);
            ps.firePropertyChange("PropertyY", "Potency_numeric","CLOGP");
            ps.firePropertyChange("PropertyX", "Potency_numeric","LC50");
            ps.firePropertyChange("PropertyX", "\"LLNA  Sis\"_2","MolWeight");
            ps.firePropertyChange("PropertyZ", "Potency_numeric","LC50");
            ps.firePropertyChange("PropertyZ", "Similarity","MolWeight");
            
            ps.firePropertyChange("DATA", null,list1);
            
            JOptionPane.showMessageDialog(null,panel);
            reader.close();
            
            */
            
            /*
            file = new File("data/200406EINECS.sdf");

            final MoleculesFile mf2 = new MoleculesFile(file, DefaultChemObjectBuilder.getInstance());
            mf2.getReader().addChemObjectIOListener(listener);
           	
           	props.clear();
           	props.add("ALogP");
           	props.add("Molecular_Weight");
           	props.add("SourceTag");
            props.add("Original_Smiles");
            
            mf2.bufferedProperties(props);
            
            m1 = new FilteredMoleculesFile(mf2,new RangePropertyFilter("ALogP",-1,0)); 
            assertTrue(m1.getAtomContainerCount()>0);
            
            
            final Subsets list1 = new Subsets();
            Subset subset = new Subset();
            subset.setParentDataset(mf2);
            list1.add(subset);
            panel = new ChemCASTChart(list1,null,
            		"ALogP",
            	    "Molecular_Weight",
            		"SourceTag",
            		null,
            		"Original_Smiles"
            		);
            
            JOptionPane.showMessageDialog(null,panel);
            mf2.close();
            */
            
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    }


}
