/* FileOpenAction.java
 * Author: Nina Jeliazkova
 * Date: Oct 1, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui.actions;

import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;

import ambit.data.descriptors.DescriptorsHashtable;
import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.PropertyTranslator;
import ambit.io.FileInputState;
import ambit.io.MolPropertiesIOSetting;
import ambit.io.MyIOUtilities;
import ambit.misc.AmbitCONSTANTS;
import ambit.ui.UITools;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.data.MoleculesFile;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Endpoint;
import chemCast.similarity.data.Subsets;

/**
 * Opens a file, creates a {@link DatasetDef} and adds to current {@link Subsets} 
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Oct 1, 2006
 */
public class FileOpenAction extends ToxmatchAction {
	protected static ImageIcon icon = UITools.createImageIcon("ambit/ui/images/open_document_16.png");
    /**
     * 
     */
    private static final long serialVersionUID = -9180414144758681172L;
    protected DescriptorsHashtable descriptorLookup = null;
    protected Subsets subsets;
    protected DatasetDef dataset;
    /**
     * @param userData
     * @param mainFrame
     * @param dataset
     */
    public FileOpenAction(ChemCASTModule userData, JFrame mainFrame, int dataset) {
        this(userData, mainFrame, "Open",dataset);
    }

    /**
     * @param userData
     * @param mainFrame
     * @param name
     * @param dataset
     */
    public FileOpenAction(ChemCASTModule userData, JFrame mainFrame,
            String name, int dataset) {
        this(userData, mainFrame, name, 
                icon,dataset);
        subsets = new Subsets();
    }

    /**
     * @param userData
     * @param mainFrame
     * @param name
     * @param icon
     * @param dataset
     */
    public FileOpenAction(ChemCASTModule userData, JFrame mainFrame,
            String name, Icon icon, int dataset) {
        super(userData, mainFrame, name, icon, dataset);
        descriptorLookup = new DescriptorsHashtable();
        subsets = new Subsets();
    }
    protected Hashtable<String,String[]> getOptions() {
        Hashtable<String,String[]> t = new Hashtable<String,String[]>(); 
        t.put(PropertyTranslator.type_identifiers,new String[] {CDKConstants.CASRN,CDKConstants.NAMES,AmbitCONSTANTS.SMILES,AmbitCONSTANTS.INCHI,""});
        DatasetDef d = getChemCastData().getDataset(ChemCASTModule.DATA_TRAINING).getParentDataset();
        if (d != null) {
            PropertyTranslator p = d.getProperties();
            Hashtable h = p.getDescriptorProperties();
            String[] option = new String[h.values().size()];
            Iterator values = h.values().iterator();
            int i=0;
            while (values.hasNext()) {
                option[i] = values.next().toString();
                i++;
            }
            t.put(PropertyTranslator.type_descriptors, option);
        }
        return t;
    }
    protected PropertyTranslator getDictionary() {
        DatasetDef d = getChemCastData().getDataset(ChemCASTModule.DATA_TRAINING).getParentDataset();
        if (d != null) {
            return d.getProperties();
        } else return null;
    }
    /* (non-Javadoc)
     * @see chemCast.ui.actions.ChemCASTAction#processDataset(chemCast.data.DatasetList)
     */
    @Override
    protected void processDataset(Subsets data) throws Exception {
        try {
             getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(false));
             dataset = getDataset();

             getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,toString());
             getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
             
             if (dataset != null) {
                //data.createSubsets(list,subsets);

              	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading " + dataset.toString());
                
               	//IAtomContainersList mf = dataset.createData(getReaderListener(),getOptions());
              	IAtomContainersList mf = dataset.createData(getReaderListener(),getDictionary());
                if (mf instanceof MoleculesFile) {
                    ArrayList<String> list = getChemCastData().getProperties();
                    for (int i=0; i < subsets.size();i++) {
                            if (subsets.get(i).getProperty("tag")!=null)
                            list.add(subsets.get(i).getProperty("tag").toString());
                    }
                        
                    getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading molecule properties from " + dataset.toString());                            
                        ((MoleculesFile)mf).bufferedProperties(list);
                }
                
                getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Loading subsets of " + dataset.toString());
              	data.createSubsets(dataset,subsets);
              	updateEndpoint(((ChemCASTModule) userData).getEndpoint(),dataset);

             } else
            	 getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,"No dataset selected!");

             getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(false));        
             getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"");        
             
         } catch (SubsetException x) {
        	 dataset = null;
             getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(false));        
             getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"");        
             	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x.getMessage());  
                    JOptionPane.showMessageDialog(mainFrame,x.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
                    
        }
         
    }
    protected void updateEndpoint(Endpoint endpoint, DatasetDef dataset) {
    	//does nothing by default
    }
    public DatasetDef getDataset() throws SubsetException {
    	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Select a file with chemicalcompounds");
        File file = MyIOUtilities.selectFile(mainFrame,null,
                getChemCastData().getDefaultDir(),
                FileInputState.extensions,FileInputState.extensionDescription,true);
        if (file == null) return null;
        else
        	return new DatasetDef(file.getAbsolutePath(),"","");
    }

    protected IReaderListener getReaderListener() {
        return new IReaderListener() {
          public void frameRead(ReaderEvent e) {
        	  getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
        	  //getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading record");
          }
          public void processIOSettingQuestion(IOSetting setting) {
              if (setting instanceof MolPropertiesIOSetting) {
            	  dataset.setProperties(((MolPropertiesIOSetting)setting).getProperties());
            	  /*
                  MolProperties props = ((ChemCASTModule) userData).getEndpoint().getTrainingData().getProperties();
                    if (identifiersProcessor == null) identifiersProcessor = new IdentifiersProcessor();
                    identifiersProcessor.addIdentifiers(props.getIdentifiers());
                    descriptorLookup.putAll(props.getDescriptors());
//                  ((MolPropertiesIOSetting)setting).getProperties().setIdentifiers(props.getIdentifiers());
                    */
              }  
          }
        };
    }
     
}
