/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.ui.wizard;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import ambit.data.IAmbitEditor;
import ambit.data.literature.ReferenceFactory;
import ambit.ui.EditorPanel;
import ambit.ui.data.HashtableModel;
import chemCast.similarity.data.ChemCastDescriptor;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.SimilarityDef;
import chemCast.similarity.data.SimilarityDefEditor;
import chemCast.similarity.data.SimilarityDefs;
import chemCast.similarity.data.SimilarityModel;
import chemCast.similarity.data.Subsets;
import chemCast.ui.RadioBoxPanel;

import com.nexes.wizard.WizardPanelDescriptor;

public class SimilarityWizard extends WizardWrapper {

	protected SimilarityDefs similarityMethods;
    protected Subsets subsets;
    RadioBoxPanel similarityPanel;
    //ListPanel subsetsPanel;
    RadioBoxPanel subsetsPanel;
    EditorPanel methodPanel;
    ChemCastWizardPanelDescriptor descriptor1;
    ChemCastWizardPanelDescriptor descriptor2;
    ChemCastWizardPanelDescriptor descriptor3;
    protected ActionMap actions;
    protected AddDescriptorsAction descriptorsAction;
    protected static String classify = "Calculate similarity and classify into groups";
    protected static String predict = "Calculate similarity and predict activity";
    
	public SimilarityWizard(Container owner,SimilarityDefs similarityMethods, SimilarityDef selected, Subsets subsets) throws Exception {
		super(owner);
        this.similarityMethods = similarityMethods;
        this.subsets = subsets;
        createPanels(similarityMethods.indexOf(selected),subsets);
	}

    protected void createPanels(int selected,Subsets subsets) throws Exception {
        actions = new ActionMap();

    	List list = similarityMethods.getList();
    	if ((list==null) || (list.size() == 0)) throw new Exception("Similarity methods not defined!");
    	
        similarityPanel = new RadioBoxPanel("Similarity methods",similarityMethods.getList(),selected){
            @Override
            public void selectObject(ActionEvent e, Object object) {
                if (object instanceof SimilarityDef) {
                    descriptorsAction.setSimilarity((SimilarityDef)object);
                	descriptor1.setNextId("similarity");
                    IAmbitEditor editor = ((SimilarityDef) object).getEditor();
                	methodPanel.setEditor(editor);
                    if (editor instanceof SimilarityDefEditor) {
                        ((SimilarityDefEditor)editor).setDescriptorActions(actions);
                    }
                }
            }
        };
        descriptorsAction = new AddDescriptorsAction(similarityPanel,subsets);
        actions.put("Add", descriptorsAction);        
        descriptor1 = new ChemCastWizardPanelDescriptor("similaritymethods","similarity",null,similarityPanel,null);
        registerWizardPanel(descriptor1.getId().toString(), descriptor1);
        
        //subsetsPanel = new ListPanel("Groups",subsets);
        List<String> options = new ArrayList<String>();
        
        //options.add("<html>Predict <b>" + subsets.getParentDataset().getDatasetResultField() + "</b></html>");
        options.add(predict);
        options.add(classify);
        selectSubsets(false);
        subsetsPanel = new RadioBoxPanel("Select action:",options,0) {
        	public void selectObject(ActionEvent e, Object object) {
        		selectSubsets(classify.equals(object));
        	};
        };
        descriptor2 = new ChemCastWizardPanelDescriptor("subsets",WizardPanelDescriptor.FINISH,"similarity",subsetsPanel,null);
        registerWizardPanel(descriptor2.getId().toString(), descriptor2);

        methodPanel = new EditorPanel();
        descriptor3 = new ChemCastWizardPanelDescriptor("similarity","subsets","similaritymethods",methodPanel,null);
        registerWizardPanel(descriptor3.getId().toString(), descriptor3);
        
        setCurrentPanel(descriptor1.getId().toString());
        if (selected<0) selected =0;
        similarityPanel.selectObject(null, similarityMethods.getList().get(selected)); 
    }
    protected void selectSubsets(boolean classes) {
		for (int i=0; i < subsets.size();i++)
			if (subsets.get(i) instanceof DatasetDef) {
				subsets.get(i).setEnabled(!classes);
			} else subsets.get(i).setEnabled(classes);
    }
    public SimilarityModel getSimilarityMethod() {
        
        int index = similarityPanel.getSelectedIndex();
        if (index >= 0) {
            int selected = 0;
            for (int i=0; i < subsets.size();i++)
                if (subsets.get(i).isEnabled()) selected++;
            int[] indices = new int[selected];
            selected = 0;
            for (int i=0; i < subsets.size();i++)
                if (subsets.get(i).isEnabled()) {
                    indices[selected] = i;
                    selected++;
                }
            SimilarityModel m = new SimilarityModel(similarityMethods.get(index));
            m.setSubsetIndices(indices);
            return m;
        }
        else return null;
        
    }

}

class AddDescriptorsAction extends AbstractAction {
    /**
     * 
     */
    private static final long serialVersionUID = 4614817675690615585L;
    protected SimilarityDef similarity = null;
    protected Subsets subsets = null;
    protected Component parent;
    public AddDescriptorsAction(Component parent,Subsets subsets) {
        super("Add");
        this.subsets = subsets;
        this.parent = parent;
        putValue(AbstractAction.SHORT_DESCRIPTION, "Add descriptors");
    }
    public void actionPerformed(ActionEvent arg0) {
        HashtableModel model = new HashtableModel(subsets.getParentDataset().getProperties().getDescriptorProperties(),false);
        JTable table = new JTable(model)  {
            
            @Override
            public String getColumnName(int arg0) {
                switch (arg0) {
                case 0: return "Selected";
                case 1: return "Name";
                default: return "";
                }
            }
            
            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return (columnIndex==0);
            }

        };
        if (JOptionPane.showConfirmDialog(parent, 
                new JScrollPane(table),"Select descriptors",JOptionPane.OK_CANCEL_OPTION)==JOptionPane.OK_OPTION) {
            if (similarity != null) {
                for (int i=0; i < model.getRowCount();i++)
                    if ((Boolean)model.getValueAt(i,0)) {
                        ChemCastDescriptorsList dlist = similarity.getDescriptors();
                        ChemCastDescriptor d = new ChemCastDescriptor(model.getValueAt(i,1).toString(),ReferenceFactory.createEmptyReference());
                        if (dlist.indexOf(d)<0)
                            dlist.add(d);
                    }    
            }
        }
        
    }
    public synchronized SimilarityDef getSimilarity() {
        return similarity;
    }
    public synchronized void setSimilarity(SimilarityDef similarity) {
        this.similarity = similarity;
    }
}

