/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import java.util.BitSet;
import java.util.Enumeration;
import java.util.Random;

import org.openscience.cdk.interfaces.IAtomContainer;

import weka.attributeSelection.AttributeEvaluator;
import weka.attributeSelection.GainRatioAttributeEval;
import weka.attributeSelection.ReliefFAttributeEval;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.lazy.IBk;
import weka.classifiers.trees.J48;
import weka.classifiers.trees.REPTree;
import weka.clusterers.ClusterEvaluation;
import weka.clusterers.Clusterer;
import weka.clusterers.EM;
import weka.clusterers.SimpleKMeans;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.AddCluster;
import ambit.data.molecule.IAtomContainersList;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.structure.FingerprintGenerator;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.Subsets;

public class DescriptorSelection {
    /*
	public static boolean selectBestDescriptorsForSubsets(ChemCastDescriptorsList descriptors, Subsets subsets) throws Exception {
		//selectBestDescriptors(new InfoGainAttributeEval(),descriptors, subsets);
		return selectBestDescriptorsForSubsets(new GainRatioAttributeEval(),descriptors,true, subsets);
	}
    public static boolean selectBestDescriptorsForDataset(ChemCastDescriptorsList descriptors, String classProperty, Subsets subsets) throws Exception {
        //ChiSquaredAttributeEval
        return selectBestDescriptorsForDataset(new ReliefFAttributeEval(),descriptors, classProperty, subsets);
    }
    */
	/**
	 * Selects descriptors that best discriminate the subsets. Only subsets and descriptors that enabled=true are 
	 * included in the evaluation! 
	 * @param descriptors
	 * @param subsets
	 */
    /*
	public static boolean selectBestDescriptorsForSubsets(AttributeEvaluator evaluator, ChemCastDescriptorsList descriptors, boolean fingerprints,  Subsets subsets) throws Exception {
		FastVector attributes = new FastVector();
		for (int i=0; i < descriptors.getList().size(); i++ )
			if (descriptors.get(i).isEnabled())
				attributes.addElement(new Attribute(descriptors.get(i).getName()));
		
		//Instances instances = subsets2Instances(descriptors,subsets);
        Instances instances = WekaSimilarityModel.properties2Instances(descriptors,null,true,subsets);
        if (instances.numInstances()==0) return false;
		evaluator.buildEvaluator(instances);
        System.out.println(evaluator);
        Enumeration en =  instances.enumerateAttributes();
        //weka.filters.unsupervised.attribute.Remove filter = new weka.filters.unsupervised.attribute.Remove();
        while (en.hasMoreElements()) {
            Attribute att = (Attribute)en.nextElement();
            double w = evaluator.evaluateAttribute((att).index());
            if (w >0) {
                System.out.print(att);
                System.out.print('\t');
                System.out.print(w);
                System.out.println();
            }
        }
        
        //AttributesSel
        //----test - use attributes for classifier
        Classifier tree = new J48();
        Clusterer clusterer = new SimpleKMeans();
        String nclasses = Integer.toString(subsets.size()-1);
        ((SimpleKMeans) clusterer).setOptions(new String[] {"-N",nclasses});
        try {
            tree.buildClassifier(instances);
            Evaluation ev = new Evaluation(instances);
            //ev.crossValidateModel(tree,instances,10,new Random());
            ev.evaluateModel(tree,instances);
            System.out.println(tree.toString());
            System.out.println(ev.toMatrixString());
            System.out.println(ev.toSummaryString());
            
            weka.filters.unsupervised.attribute.Remove filter = new weka.filters.unsupervised.attribute.Remove();
            weka.filters.unsupervised.attribute.AddCluster cFilter = new AddCluster();
            filter.setAttributeIndices("" + (instances.classIndex() + 1));
            filter.setInputFormat(instances);
            Instances dataClusterer = Filter.useFilter(instances, filter);

            clusterer.buildClusterer(dataClusterer);
            ClusterEvaluation cev = new ClusterEvaluation();
            cev.setClusterer(clusterer);
            cev.evaluateClusterer(instances);
            System.out.println(cev.clusterResultsToString());
        } catch (Exception x) {
            x.printStackTrace();
        }
        //--
		int a = 0;
		for (int i=0; i < descriptors.getList().size(); i++ )
			if (descriptors.get(i).isEnabled()) {
				descriptors.get(i).setWeight(evaluator.evaluateAttribute(a));
				a++;
			}	
		instances = null;
        return true;
	}
    public static boolean selectBestDescriptorsForDataset(AttributeEvaluator evaluator, ChemCastDescriptorsList descriptors, String classProperty, Subsets subsets) throws Exception {
        FastVector attributes = new FastVector();
        for (int i=0; i < descriptors.getList().size(); i++ )
            if (descriptors.get(i).isEnabled())
                attributes.addElement(new Attribute(descriptors.get(i).getName()));
        
        //Instances instances = subsets2Instances(descriptors,subsets);
        Instances instances = WekaSimilarityModel.properties2Instances(descriptors,classProperty,false,subsets);
        if (instances.numInstances()==0) return false;
        evaluator.buildEvaluator(instances);
        
        System.out.println(evaluator);
        int a = 0;
        for (int i=0; i < descriptors.getList().size(); i++ )
            if (descriptors.get(i).isEnabled()) {
                descriptors.get(i).setWeight(evaluator.evaluateAttribute(a));
                a++;
            }
        
        //----test - use attributes for classifier
        
        Classifier tree = new REPTree();
        try {
            tree.buildClassifier(instances);
            Evaluation ev = new Evaluation(instances);
            ev.crossValidateModel(tree,instances,10,new Random());
            //ev.evaluateModel(tree,instances);
            System.out.println(tree.toString());
            //System.out.println(ev.toMatrixString());
            System.out.println(ev.toSummaryString());
            
        } catch (Exception x) {
            x.printStackTrace();
        }
        
        //--

        instances = null;
        return true;
    }
    
    */
    
    
}



