/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import weka.core.Instance;
import weka.core.Instances;

/**
 * Dice coefficient or Hodgkin-Richards Index.
 * Dice coefficient = 2*Sum(Xsj*Xtj)/[Sum(Xsj*Xsj)^2+Sum(Xtj*Xtj)^2] 
 * or Hodgkin-Richards similarity index is Cab=2*Zab/(Zaa+Zbb). 
 * This class calculates the dissimilarity 1-Cab.
 * See  Todeschini,Consonni, "Handbook of Molecular Descriptors", p.399.
 * @author Nina Jeliazkova
 *
 */
public class DiceSimilarity extends AbstractSimilarity {

	/**
	 * 
	 */
	private static final long serialVersionUID = -4991499564931056797L;
	public DiceSimilarity() {
		super();
	}

	public DiceSimilarity(Instances data) {
		super(data);

	}
    public float getDistance(Instance a, Instance b) throws Exception {
        // TODO Auto-generated method stub
        return (float) distance(a, b,Float.MAX_VALUE);
    }

    public float getNativeComparison(Instance a, Instance b) throws Exception {
        double zab = zAB(a,b);
        
        double d = zAA(a)+zAA(b);
        if (d != 0)
            return  (float) (2*zab/d);
        else
            if (zab == 0) return Float.MAX_VALUE; //0/0
            else return 0; //infinity
    }    
	@Override
	public double distance(Instance a, Instance b) throws Exception {
		return distance(a,b,Double.MAX_VALUE,false);
	}
	@Override
	public double distance(Instance a, Instance b, double arg2) {
		return distance(a, b, arg2, false);
	}
	@Override
	public double distance(Instance a, Instance b, double arg2, boolean print) {
        double zab = zAB(a,b);
        
		double d = zAA(a)+zAA(b);
		if (d != 0)
		    return  similarity2distance(2*zab/d);
		else
            if (zab == 0) return 0; //0/0
            else return arg2; //infinity
	}	
	@Override
	/**
	 * does nothing
	 */
	public void postProcessDistances(double[] distances) {
	    
	}
	@Override
	public String toString() {
        return "Dice coefficient = 2*Sum(Xsj*Xtj)/[Sum(Xsj*Xsj)^2+Sum(Xtj*Xtj)^2]"; 
        
	}




}


