/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.io;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

import org.openscience.cdk.ChemObject;
import org.openscience.cdk.DefaultChemObjectBuilder;
import org.openscience.cdk.SetOfMolecules;
import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.interfaces.IChemFile;
import org.openscience.cdk.interfaces.IChemObject;
import org.openscience.cdk.interfaces.IMolecule;
import org.openscience.cdk.interfaces.ISetOfMolecules;
import org.openscience.cdk.io.DefaultChemObjectWriter;
import org.openscience.cdk.io.formats.IResourceFormat;
import org.openscience.cdk.smiles.SmilesGenerator;
import org.openscience.cdk.tools.DataFeatures;
import org.openscience.cdk.tools.LoggingTool;

/**
 * TODO add description
 * @author ThinClient
 * <b>Modified</b> 2005-9-6
 */
public class DelimitedFileWriter extends DefaultChemObjectWriter {
	protected static LoggingTool logger = new LoggingTool(DelimitedFileWriter.class);
	protected BufferedWriter writer;
	protected DelimitedFileFormat format;
	protected ArrayList header = null;
	protected int smilesIndex = -1;
	protected SmilesGenerator sg = new SmilesGenerator(DefaultChemObjectBuilder.getInstance());
	public static String defaultSMILESHeader = "SMILES";
	protected boolean writingStarted = false;
	/**
	 * 
	 */
    public DelimitedFileWriter(Writer out, DelimitedFileFormat format) {
        //logger = new LoggingTool(this);
    	this.format = format;
        try {
            writer = new BufferedWriter(out);
        } catch (Exception exc) {
        }
    }
    public DelimitedFileWriter(Writer out) {
    	this(out,new DelimitedFileFormat());
    }
    public DelimitedFileWriter(OutputStream input, DelimitedFileFormat format) {
        this(new OutputStreamWriter(input), format);
    }
    public DelimitedFileWriter(OutputStream input) {
    	this(input, new DelimitedFileFormat());
    }
	/* (non-Javadoc)
	 * @see org.openscience.cdk.io.ChemObjectWriter#write(org.openscience.cdk.ChemObject)
	 */
	public void write(IChemObject object) throws CDKException {
		
		if (object instanceof ISetOfMolecules) {
		    writeSetOfMolecules((ISetOfMolecules)object);
		} else if (object instanceof IMolecule) {
		    writeMolecule((IMolecule)object);
		} else {
		    throw new CDKException("Only supported is writing of ChemFile and Molecule objects.");
		}

	}

	/* (non-Javadoc)
	 * @see org.openscience.cdk.io.ChemObjectWriter#highestSupportedChemObject()
	 */
	public ChemObject highestSupportedChemObject() {
        return new SetOfMolecules();
	}

    public IResourceFormat getFormat() {
        return format;
    }


	/* (non-Javadoc)
	 * @see org.openscience.cdk.io.ChemObjectIO#close()
	 */
	public void close() throws IOException {
        writer.flush();
        writer.close();

	}
	public void  writeSetOfMolecules(ISetOfMolecules som)
	{
		IMolecule[] molecules = som.getMolecules();
		writeMolecule(molecules[0]);
		for (int i = 1; i <= som.getMoleculeCount() - 1; i++) {
			try {
				writeMolecule(molecules[i]);
			} catch (Exception exc) {
			}
		}
	}
	protected void writeHeader() throws IOException {
        char fieldDelimiter = format.getFieldDelimiter();
        char textDelimiter = format.getTextDelimiter();		
		for (int i=0;i<header.size();i++) {
			String h = header.get(i).toString();
			if (h.indexOf(fieldDelimiter) >  -1) {
				writer.write(textDelimiter);
				writer.write(h);
				writer.write(textDelimiter);
			} else writer.write(h);
			writer.write(fieldDelimiter);			
		}
		writer.newLine();
		logger.debug(format.getFormatName(),"\tHeader written\t",header);
	}
    public void writeMolecule(IMolecule molecule) {
        
        char fieldDelimiter = format.getFieldDelimiter();
        char textDelimiter = format.getTextDelimiter();
        Object value;    	

        try {
        	//give it a chance to create a header just before the first write
        	if (!writingStarted) {
    	        if (header == null) setHeader(molecule.getProperties());
    	        writeHeader();
    	        writingStarted = true;
        	}
        	String s;
        	for (int i =0; i< header.size(); i++) {
        		value = molecule.getProperty(header.get(i));
        		if (i == smilesIndex) {
        			
        			if (value == null) //no SMILES available
        			try {
        				value = sg.createSMILES(molecule);
        			} catch (Exception x) {
        				logger.error("Error while createSMILES\t",x.getMessage());
        				value = "";
        			}
        		} 
        	
        		if (value != null) {
        			if (value instanceof Number) {
        				writer.write(value.toString());
        			} else {
        				s = value.toString();
        				if (s.indexOf(textDelimiter) > -1) s=s.replace(textDelimiter,' ');
        				s = s.trim();
        				if (s.indexOf(fieldDelimiter) > -1) {
	        				writer.write(textDelimiter);
	        				writer.write(s);
	        				writer.write(textDelimiter);        
        				} else writer.write(s); 
        			}
        		}
        		if (i< (header.size()-1))
        				writer.write(fieldDelimiter);
        	}
            writer.newLine();
            writer.flush();
            logger.debug("file flushed...");
        } catch(Exception x) {
            logger.error("ERROR while writing Molecule: ", x.getMessage());
            logger.debug(x);
            x.printStackTrace();
        }
    }

	/**
	 * @return Returns the header.
	 */
	public synchronized ArrayList getHeader() {
		return header;
	}
	/**
	 * @param header The header to set.
	 */
	public synchronized void setHeader(ArrayList header) {
		if (writingStarted) {
			logger.error("Can't change header while writing !!!!");
			return; //cant' change header !
		}
		this.header = header;
		smilesIndex = -1;
		for (int i=0; i < header.size(); i++) 
			if (header.get(i).equals(defaultSMILESHeader)) smilesIndex = i;
		if (smilesIndex == -1) { header.add(0,defaultSMILESHeader); smilesIndex = 0; }
		logger.info("Header created\t",header);
	}
	/**
	 * Creates header from Hashtable keys
	 * Used for default header - created from properties of the first molecule written 
	 * @param properties
	 */
	public void setHeader(Hashtable properties) {
		if (writingStarted) {
			logger.error("Can't change header while writing !!!!");
			return; //cant' change header !
		}		
		header = new ArrayList();
		Enumeration e = properties.keys();
		smilesIndex = -1; int i = 0;
		while (e.hasMoreElements()) {
			header.add(e.nextElement());
			if (header.get(i).equals(defaultSMILESHeader)) smilesIndex = i;
			i++;
		}
		if (smilesIndex == -1) { header.add(0,defaultSMILESHeader); smilesIndex = 0; }
		logger.info("Header created from hashtable\t",header);
	}
    @Override
	public String toString() {
        return "Writing compounds to " + format.toString(); 
    }
	public boolean accepts(Class classObject) {
		Class[] interfaces = classObject.getInterfaces();
		for (int i=0; i<interfaces.length; i++) {
			if (IChemFile.class.equals(interfaces[i])) return true;
			if (ISetOfMolecules.class.equals(interfaces[i])) return true;
		}
		return false;
	}
	public int getSupportedDataFeatures() {
		return DataFeatures.HAS_GRAPH_REPRESENTATION;
	}
	/* (non-Javadoc)
     * @see org.openscience.cdk.io.IChemObjectWriter#setWriter(java.io.OutputStream)
     */
    public void setWriter(OutputStream writer) throws CDKException {
        this.setWriter(new OutputStreamWriter(writer));

    }
    /* (non-Javadoc)
     * @see org.openscience.cdk.io.IChemObjectWriter#setWriter(java.io.Writer)
     */
    public void setWriter(Writer writer) throws CDKException {
        if (this.writer != null) {
            try {
            this.writer.close();
            } catch (IOException x) {
                logger.error(x);
            }
            this.writer = null;
        }
        this.writer = new BufferedWriter(writer);

    }
}
