/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.test.tree.cramer;

import junit.framework.TestCase;

import org.openscience.cdk.Molecule;

import toxTree.core.IDecisionRule;
import toxTree.exceptions.DecisionMethodException;
import toxTree.exceptions.MolAnalyseException;
import toxTree.logging.TTLogger;
import toxTree.query.FunctionalGroups;
import toxTree.query.MolAnalyser;
import toxTree.tree.cramer.RuleSimplyBranchedAliphaticHydrocarbon;

/**
 * A tets for Cramer rule No 5
 * @author Nina Jeliazkova
 * <b>Modified</b> 2005-8-24
 */
public class RuleSimplyBranchedAliphaticHydrocarbonTest extends TestCase {
	public static TTLogger logger = new TTLogger(RuleSimplyBranchedAliphaticHydrocarbonTest.class);
	public static void main(String[] args) {
		junit.textui.TestRunner
				.run(RuleSimplyBranchedAliphaticHydrocarbonTest.class);
	}

	/*
	 * @see TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	/*
	 * @see TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Constructor for RuleSimplyBranchedAliphaticHydrocarbonTest.
	 * @param arg0
	 */
	public RuleSimplyBranchedAliphaticHydrocarbonTest(String arg0) {
		super(arg0);
		TTLogger.configureLog4j(true);
	}
	public void testRule() {
	    IDecisionRule rule5 = new RuleSimplyBranchedAliphaticHydrocarbon();
	    //
	    Object[][] answer = {
	            {"CCCCCCCCCCC",new Boolean(true)},  
	            {"OC1COC(O)C(O)C1(O)",new Boolean(true)},
	            {"O=C1OC(=CC(=O)C1C(=O)C)C",new Boolean(false)},
	            {"CC(=O)C1=C(O)C=C(C)OC1=O",new Boolean(false)},
				
	            
	            };
	    Molecule m = (Molecule) FunctionalGroups.createAtomContainer((String)answer[2][0]);
	    try {
	    MolAnalyser.analyse(m);
	    } catch (MolAnalyseException  x) {
	    	fail();
	    }
	    assertTrue(FunctionalGroups.hasGroup(
	    		m,
				FunctionalGroups.lactone(false)));
	    
	    for (int i = 2; i < answer.length; i++) {
	        Molecule mol = (Molecule) FunctionalGroups.createAtomContainer((String)answer[i][0]);
	        if (mol != null)
	            try {
	            	MolAnalyser.analyse(mol);
	                Boolean b = (Boolean) answer[i][1];
	                assertEquals(rule5.verifyRule(mol),b.booleanValue());
	            } catch (DecisionMethodException x) {
	                x.printStackTrace();
	            } catch (MolAnalyseException x) {
	                x.printStackTrace();
	            }
	    }

	}

}
